@@ -1,5 +1,5 @@
 /*
- * $snafu: gpsversion.c,v 2.0 2003/10/06 19:13:53 marc Exp $
+ * $snafu: gpsversion.c,v 2.1 2007/04/03 17:48:59 marc Exp $
  *
  * Public Domain, 2001, Marco S Hyman <marc@snafu.org>
  */
@@ -13,14 +13,14 @@
 #include "gpslib.h"
 
 /*
- * Request the unit version and output to stdout.  Wait for a
+ * Request the unit version and possibly output to stdout.  Wait for a
  * protocol capabilities packet and consume it if seen.
  *
  *	-1:	command failed
  *	1:	command succeeded.
  */
 int
-gps_version(gps_handle gps)
+gps_version(gps_handle gps, int print)
 {
 	int product_id;
 	int software_version;
@@ -31,8 +31,10 @@ gps_version(gps_handle gps)
 		gps_printf(gps, 1, "%s: failed\n", __func__);
 		return -1;
 	}
-	printf("[product %d, version %d: %s]\n", product_id, software_version,
-	       product_description ? product_description : "unknown");
+	if (print)
+		printf("[product %d, version %d: %s]\n",
+		       product_id, software_version,
+		       product_description ? product_description : "unknown");
 	if (product_description)
 		free(product_description);
 
